package service

import (
	"context"

	"gitea.dwysokinski.me/twhelp/dcbot/internal/domain"
)

type Coords struct {
	twhelpSvc TWHelpService
}

func NewCoords(twhelpSvc TWHelpService) *Coords {
	return &Coords{twhelpSvc: twhelpSvc}
}

func (c *Coords) Translate(ctx context.Context, params domain.TranslateCoordsParams) (domain.TranslateCoordsResult, error) {
	perPage := params.PerPage()
	if coordsLen := int32(len(params.Coords())); coordsLen < perPage {
		perPage = coordsLen
	}

	coords := params.Coords()[:perPage]

	villages, err := c.twhelpSvc.ListVillagesByCoords(ctx, params.VersionCode(), params.ServerKey(), coords, 0, perPage)
	if err != nil {
		return domain.TranslateCoordsResult{}, err
	}

	untranslated := make([]string, 0, perPage)
	for _, co := range coords {
		found := false

		for _, v := range villages {
			if co == v.Coords() {
				found = true
				break
			}
		}

		if !found {
			untranslated = append(untranslated, co)
		}
	}

	return domain.TranslateCoordsResult{
		Villages:     villages,
		Page:         1,
		MaxPage:      params.MaxPage(),
		Untranslated: untranslated,
	}, nil
}
